<?php

/**
 * File upload component
 * Example at module=example&action=main:controls:upload
 *
 * TODO: error handling
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 * @author Ely Edison Matos [ely.matos@ufjf.edu.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2011/02/14
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2011 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */

class MFileField extends MTextField
{
    const UPLOAD_INFO = 'uploadInfo';

    public function __construct($name='', $value='', $label='', $size=40, $hint='')
    {
        parent::__construct($name, $value, $label, $size, $hint);
        $this->type = 'file';
        $this->setClass('mFileField');

        $this->page->addDojoRequire("dojo.parser");
        $this->page->addDojoRequire("dojo.io.iframe");
        $this->page->onLoad("miolo.page.fileUpload = 'yes';");
    }

    public function generateInner()
    {
        parent::generateInner();
        $hidden = new MHiddenField('__ISFILEUPLOADPOST', 'yes');
        $this->inner .= $hidden->generate();
    }

    /**
     * @return array List with the names of the uploaded files
     */
    public static function getUploadInfo()
    {
        $uploadInfo = MIOLO::_REQUEST(self::UPLOAD_INFO);
        if ( $uploadInfo )
        {
            $uploadInfo = explode(',', $uploadInfo);
        }

        return $uploadInfo;
    }

    /**
     * Move the uploaded files to the specified path
     *
     * @param string $path Path where the files must be stored
     * @return array Array with name and path of the uploaded files
     */
    public static function uploadFiles($path)
    {
        $MIOLO = MIOLO::getInstance();
        $uploadInfo = self::getUploadInfo();

        if ( !is_array($uploadInfo) )
        {
            return;
        }

        $uploadedFiles = array();

        foreach ( $uploadInfo as $fileInfo )
        {
            list($fileName, $tmpFile) = explode(';', $fileInfo);

            $tmpFile = $MIOLO->getConf('home.html') . "/files/tmp/$tmpFile";
            if ( copy($tmpFile, "$path/$fileName") )
            {
                unlink($tmpFile);
                $uploadedFiles[] = array(
                    'name' => $fileName,
                    'path' => "$path/$fileName"
                );
            }
        }

        return $uploadedFiles;
    }
}
?>