<?php

/**
 * MLookupContainer
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2010/08/04
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2010 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */
class MLookupContainer extends MContainer
{
    public $value;
    public $descriptionSize;
    public $label;
    public $item;
    public $module;
    public $hint;

    private $lookupField;
    private $descriptionField;

    public function __construct($name, $value = '', $label = '', $module=null, $item='', $readOnly=false, $hint='', $descriptionSize = 50)
    {
        $descriptionId = $name . '_lookupDescription';

        $this->lookupField = new MLookupTextField($name, $value, NULL, 10);
        $this->descriptionField = new MTextField($descriptionId, NULL, NULL, $descriptionSize, $hint, NULL, true);

        $this->label = $label;
        $this->module = $module ? $module : MIOLO::getCurrentModule();
        $this->item = $item;

        $this->lookupField->setContext($this->module, $this->module, $this->item, 'filler', "$name,$descriptionId", NULL, true);
        $this->lookupField->setReadOnly($readOnly);

        $controls = array( $this->lookupField, $this->descriptionField );

        parent::__construct($name . '_container', $controls, 'horizontal', MFormControl::FORM_MODE_SHOW_NBSP);
    }

    public function getLookupField()
    {
        return $this->lookupField;
    }

    public function setLookupField($lookupField)
    {
        $this->lookupField = $lookupField;
    }

    public function getDescriptionField()
    {
        return $this->descriptionField;
    }

    public function setDescriptionField($descriptionField)
    {
        $this->descriptionField = $descriptionField;
    }

    public function setContext($module, $item, $related, $filter, $autoComplete = true)
    {
        $this->lookupField->setContext($module, $module, $item, 'filler', $related, $filter, $autoComplete);
    }

    public function getRelated()
    {
        return $this->lookupField->related;
    }

    public function doAutoComplete($value = null)
    {
        $_POST['filter'] = $value ? $value : $this->lookupField->value;

        if ( $this->lookupField->value || $value )
        {
            $bussinesPath = $this->manager->getConf('namespace.business'); //  <business>/classes</business>

            $this->manager->uses( $bussinesPath.'/lookup.class.php', $this->lookupField->baseModule );

            $lookupName = "Business{$this->lookupField->baseModule}Lookup";
            $autoCompleteName = "AutoComplete{$this->item}";

            $lookup = new $lookupName();
            $mlookup = new MLookup($this->lookupField->baseModule);

            if ( method_exists( $lookup, $autoCompleteName ) )
            {
                $result = $lookup->$autoCompleteName($mlookup);
            }

            if ( !$value )
            {
                $related = explode(',', $this->lookupField->related );

                if ( is_array( $related ) && is_array( $result ) )
                {
                    foreach ( $related as $line => $id )
                    {
                        $this->page->setElementValue($id, $result[$line]);
                    }
                }
            }

            return $result;
        }
    }

    public function getContextAttribute($attribute)
    {
        return $this->lookupField->$attribute;
    }

    public function setReadOnly($readOnly)
    {
        $this->lookupField->setReadOnly($readOnly);
    }

    public function generateLabel()
    {
        $label = '';
        if ( $this->label != '' )
        {
            $span  = new MSpan( '', $this->label . ':', 'label' );

            $r = $this->attrs->items['required'] || ($this->validator && $this->validator->type == 'required');

            if ( $r && trim(MUtil::removeSpaceChars($this->label)) )
            {
                $span->setClass('mCaptionRequired');
            }

            $label = $this->painter->span( $span );
        }

        return $label;
    }

    public function generate()
    {
        $span = new MSpan(NULL, parent::generate(), 'field');

        return $this->generateLabel() . $span->generate();
    }

    public function addAttribute($attr, $value)
    {
        $this->lookupField->addAttribute($attr, $value);
    }

    public function addEvent($event, $value, $preventDefault = true)
    {
        $this->lookupField->addEvent($event, $value, $preventDefault);
    }

    public function setValue($value)
    {
        $this->lookupField->setValue($this->value);
    }

    public function getValue()
    {
        return $this->value;
    }

    public function setAutoComplete($autocomplete=true)
    {
        $this->lookupField->setAutoComplete($autocomplete);
    }
}
?>
