<?php

/**
 * MSpecialGrid component.
 * A MGrid component with some extra features, like row selection by clicking
 * and JS hiding columns.
 *
 * @author Armando Taffarel Neto [taffarel@solis.coop.br]
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2010/08/17
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2010 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */

$MIOLO = MIOLO::getInstance();
$MIOLO->page->addScript('m_specialgrid.js');

class MSpecialGrid extends MGrid
{
    public $module;
    public $arguments;
    public $selectsData;
    public $showVisibleColumnsMenu;
    public $invisibleColumns;

    /**
     * MSpecialGrid
     *
     * @param (array) $data Data to fill the grid
     * @param (array) $columns MGridColumn array
     * @param (string) $name Id
     * @param (int) $pageLength Number of lines per page
     * @param (boolean) $showCheckBoxes Whether the the check boxes must be shown
     * @param (array) $arguments Array with the names for the selected lines in post
     */
    public function __construct($data, $columns, $name, $pageLength = 25, $showCheckBoxes = true, $arguments = NULL)
    {
        $this->showVisibleColumnsMenu = true;
        $this->module = MIOLO::getCurrentModule();

        parent::__construct($data, $columns, MIOLO::getCurrentURL(), $pageLength, 0, $name, true, true);

        if ( $showCheckBoxes )
        {
            $this->select = new MSpecialGridActionSelect($this);
        }

        $this->arguments = $arguments ? $arguments : array( 'id' => '%0%' );

        // Control and shift keys selection
        $this->selectKeys($this->id);
    }

    /**
     * Returns the selected rows
     *
     * @return (array) Array with the selected rows
     */
    public function getSelectsData($name = null)
    {
        if ( !$name )
        {
            $name = $this->name;
        }

        $selecteds = $this->page->request('select' . $name);

        return self::getSelectedAsIndexedArray($selecteds);
    }

    /**
     * Defines the check boxes to be checked
     *
     * @param (array) $selectedData Array with the rows to be checked
     */
    public function setSelectedData($selectedData)
    {
        $this->selectedData = $selectedData;
    }

    /**
     * Returns selected rows as an indexed array
     *
     * @param (array) $selectedRows Post result for selected rows
     * @return (array) Object array
     */
    public static function getSelectedAsIndexedArray($selectedRows)
    {
        foreach ( $selectedRows as $selected )
        {
            foreach ( explode('&', $selected) as $argument )
            {
                if ( $argument )
                {
                    list ($key, $value) = explode('|', $argument);
                    $line[$key] = $value;
                }
            }
            $data[] = $line;
        }

        return $data;
    }

    /**
     * Removes the grid check boxes
     *
     */
    public function removeSelects()
    {
        $this->select = null;
    }

    /**
     * Sets an indexed array to be returned by the getSelectsData method
     *
     * @param (array) $arguments Indexed array like: array('id' => '%0%')
     */
    public function setArguments($arguments)
    {
        $this->arguments = $arguments;
    }

    public function setInvisibleColumns($invisibleColumns)
    {
        $this->invisibleColumns = $invisibleColumns;
    }

    /**
     * Overrides grid navigation header method
     *
     * @param (boolean) $showVisibleColumnsMenu 
     * @return (object) MDiv
     */
    public function generateNavigationHeader($showVisibleColumnsMenu = true)
    {
        $MIOLO = MIOLO::getInstance();
        if ( $this->pn )
        {
            $array[0] = new MSpan('', _M('Page') . ':');
            $array[0]->addStyle('margin-left', '10px');
            $array[0]->addStyle('margin-right', '5px');

            if ( $this->pn->getPageCount() > 1 && ( $this->pn->getPageNumber() != 1 ) )
            {
                $array[1] = $this->pn->getPageFirst();
                $array[1]->setLabel(_M('First'));

                $array[2] = $this->pn->getPagePrev();
                $array[2]->setLabel(_M('Previous'));
            }

            $pages = $this->pn->getPageLinks(false);
            foreach ( $pages->getInner() as $page )
            {
                $page->setLabel('[' . $page->label . ']');
                $newPages[] = $page;
            }
            $pages = new MSpan('', $newPages);
            $pages->addStyle('margin-right', '10px');
            $array[3] = $pages;

            if ( $this->pn->getPageCount() > 1 && ( $this->pn->getPageNumber() != $this->pn->getPageCount() ) )
            {
                $array[4] = $this->pn->getPageNext();
                $array[4]->setLabel(_M('Next'));

                $array[5] = $this->pn->getPageLast();
                $array[5]->setLabel(_M('Last'));
            }

            $content[] = new MDiv('', $array, 'mPageNavigator');
        }

        // FIXME: Uncomments this when the generateVisibleColumns method is ready
        /*if ( $showVisibleColumnsMenu && $this->showVisibleColumnsMenu )
        {
            $content[] = $this->generateVisibleColumns();
        }*/

        if ( $content )
        {
            return new MDiv('', $content, 'mGridNavigation');
        }
    }

    public function setHeaderColumns($columns)
    {
        if ( is_array($columns) )
        {
            foreach ( $columns as $column )
            {
                $jsCode .= "dojo.forEach(dojo.byId('row{$this->name}[$column]').childNodes, function (element) { if (element.style) element.className ='mGridHeader' } );";
            }
        }
        else
        {
            $jsCode = "dojo.forEach(dojo.byId('row{$this->name}[$columns]').childNodes, function (element) { if (element.style) element.className ='mGridHeader' } );";
        }
        $this->page->onload($jsCode);
    }

    public function generateNavigationFooter()
    {
        return $this->generateNavigationHeader(false);
    }

    public function selectKeys($id)
    {
        $MIOLO = MIOLO::getInstance();
        $MIOLO->page->onload("dojo.connect(dojo.byId('{$id}'), 'onclick', mspecialgrid.selectKeys)");
    }

    /**
     * Returns the icon to alternate the column visibility.
     *
     * FIXME: Uncomment and test this method when the MContextMenu component
     * is created.
     *
     * @return (object) MDiv containing the menu icon and the hidden menu
     */
    /*public function generateVisibleColumns()
    {
        $div = new MDiv($this->name . '_columnsDiv', '', 'mGridVisibleColumns');
        $menu = new MContextMenu($this->name . '_columnsMenu');

        $index = 0;

        foreach ( $this->columns as $key => $col )
        {
            $visible = $col->visible;

            if ( $visible )
            {
                $icon = 'mGridVisibleColumnsChecked';
                $action = "mspecialgrid.showHideColumn('{$this->id}', '{$index}');";
                $menu->addCustomItem($col->title, $action, $icon);
                $index++;
            }
        }
        $menu->setLeftClickOpen(true);
        $menu->setTarget($div);

        return new MDiv('', array( $menu, $div ));
    }*/

    public function generateHeader()
    {
        $header[] = $this->generateFilter();
        if ( $this->data )
        {
            $header[] = $this->generateNavigationHeader();
        }
        $header[] = $this->generateLinks();

        return $header;
    }

    public function generate()
    {
        $this->generateData();
        $header = $this->painter->generateToString($this->generateHeader());
        $body = $this->painter->generateToString($this->generateBody());
        $footer = $this->painter->generateToString($this->generateFooter());

        //$invisibleColumns = new MTextField($this->name . '_invisibleColumns');
        //$invisibleColumns->addStyle('display', 'none');

        $this->setInner( array($header, $body, $footer, $invisibleColumns) );
        $this->setClass('mGrid');

        if ($this->width != '')
        {
            $this->addStyle('width', $this->width);
        }

        /*
         * FIXME: removes the cells and columns size computation
         * analyze the possibility of removing it from MIOLO
         */
        unset($this->page->onload->items[$this->page->onload->find("miolo.grid.ajustTHead();")]);
        unset($this->page->onload->items[$this->page->onload->find("miolo.grid.ajustSelect('linkbtn');")]);

        // FIXME: uncomment when the MMenuContext is ready
        // Hide the columns that was hidden before by js
        /*$iColumns = $this->invisibleColumns;

        //if ( !AUtil::primeiroAcessoAoForm() && AUtil::buscaValorEventoPadrao() != 'botao_limpar_click' && MIOLO::_REQUEST( $this->name . '_invisibleColumns' ) )
        {
            $iColumns = MIOLO::_REQUEST($this->name . '_invisibleColumns');
        }

        // Using isset to consider zero string ('0') a valid value
        if ( isset($iColumns) )
        {
            $js = '';
            $invisibleColumns = explode(',', trim($iColumns, ','));
            foreach ( $invisibleColumns as $column )
            {
                $js .= "mspecialgrid.showHideColumn('{$this->id}', '{$column}');";
            }

            $this->page->onload($js);
        }*/

        $this->generateInner();
        $this->generateEvent();
        return $this->getRender('div');
    }

}

class MSpecialGridActionSelect extends MGridActionSelect
{

    public function __construct($grid, $index)
    {
        parent::__construct($grid);
    }

    public function generate()
    {
        $i = $this->grid->currentRow;
        $row = $this->grid->data[$i];

        foreach ( $this->grid->arguments as $key => $argument )
        {
            if ( strpos($argument, '%') === 0 )
            {
                // gets the column number expressed between %%
                $value = $row[substr($argument, 1, strpos($argument, '%', 1) - 1)];
            }
            else
            {
                $value = $argument;
            }

            $index .= "{$key}|{$value}&";
        }

        $control = new MCheckBox("select" . $this->grid->name . "[$i]", $index, '');
        $control->addAttribute('onclick', "miolo.grid.check(this,'" . $this->grid->name . "[$i]" . "');", false);
        if ( $this->grid->selectsData[$i] )
        {
            $control->addAttribute('checked');
        }
        return $control;
    }

}
?>
