<?php

/**
 * Component to group fields in tabs
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2010/07/20
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2010 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */

$MIOLO = MIOLO::getInstance();
$MIOLO->page->addScript('m_tabbedbasegroup.js');

class MTabbedBaseGroup extends MBaseGroup
{
    private $tabs;

    /**
     * MTabbedBaseGroup constructor
     *
     * @param string $name Id and name of the component
     * @param array $tabs Array of MTab objects
     */
    public function __construct($name, $tabs = array())
    {
        $controls[] = new MDiv("{$name}DivResponse");
        parent::__construct($name);
        $this->tabs = $tabs;
        $this->setClass('mTabbedBaseGroup');
    }

    /**
     * Creates a tab and adds it to the container
     *
     * @param string $id Id of the tab
     * @param string $label Label of the tab
     * @param array $controls Fields of the tab
     * @param string $ajaxAction An ajax action to be called when the tab is selected
     * @param boolean $disabled Informs whether the tab is disabled or not
     */
    public function createTab($id, $label, $controls = array(), $ajaxAction = NULL, $disabled = false)
    {
        $tab = new MTab($id, $label, $controls, $ajaxAction, $disabled);
        $tab->setWidth('100%');
        $tab->addStyle('display', 'none');
        $this->tabs[$id] = $tab;
    }

    /**
     * Statically creates a tab and adds it to the container
     * Usefull for ajax requests
     *
     * Note: Use it only if you really need it. If not in an ajax request,
     * you should use the createTab method instead.
     *
     * @param string $tabbedBaseGroupId Id of the MTabbedBaseGroup
     * @param string $id Id of the tab
     * @param string $label Label of the tab
     * @param array $controls Fields of the tab
     * @param string $ajaxAction An ajax action to be called when the tab is selected
     * @param boolean $disabled Informs whether the tab is disabled or not
     */
    public static function createStaticTab($tabbedBaseGroupId, $id, $label, $controls = array(), $ajaxAction = NULL, $disabled = false)
    {
        $MIOLO = MIOLO::getInstance();

        $tab = new MTab($id, $label, $controls, $ajaxAction, $disabled);
        $tab->setWidth('100%');
        $tab->addStyle('display', 'none');

        $button = self::getTabButton($tab, $tabbedBaseGroupId);
        $bGenerate = $button->generate();
        $bGenerate = str_replace("\n", '\n', $bGenerate);
        $bGenerate = str_replace("'", "\'", $bGenerate);

        $dGenerate = $tab->generate();
        $dGenerate = str_replace("\n", '\n', $dGenerate);
        $dGenerate = str_replace("'", "\'", $dGenerate);

        $js = "mtabbedbasegroup.createTab('$id', '$tabbedBaseGroupId', '$bGenerate', '$dGenerate');";

        $MIOLO->page->onload($js);
        $MIOLO->ajax->setResponse(NULL, "{$tabbedBaseGroupId}DivResponse");
    }

    /**
     * @return array Array of MTab objects
     */
    public function getTabs()
    {
        return $this->tabs;
    }

    /**
     * @param array Array of MTab objects
     */
    public function setTabs($tabs = array())
    {
        $this->tabs = $tabs;
    }

    /**
     * @param MTab Tab to be added on tabs array
     */
    public function addTab($tab)
    {
        $this->tabs[$tab->id] = $tab;
    }

    /**
     * @param string Id of the tab to be getted from tabs array
     */
    public function getTab($tabId)
    {
        return $this->tabs[$tabId];
    }

    /**
     * Generates the tab buttons
     *
     * @return object MHContainer
     */
    protected function getTabButton($tab, $tabbedBaseGroupId, $selected = false)
    {
        $MIOLO = MIOLO::getInstance();

        $tempDiv = new MDiv($tab->id . 'Button', $tab->label);

        //controla se é por ajax ou aba estatica, ou desabilita
        if ( !$tab->disabled )
        {
            $tempDiv->setClass('mTab mTabIdle');

            if ( !$tab->ajaxAction )
            {
                $tempDiv->addAttribute('onclick', "mtabbedbasegroup.changeTab('{$tab->id}', '$tabbedBaseGroupId');");
            }
            else
            {
                $url = $MIOLO->getUI()->getAjax($tab->ajaxAction);
                $tempDiv->addAttribute('onclick', "$url; mtabbedbasegroup.changeTab('{$tab->id}','$tabbedBaseGroupId');");
            }

            if ( $selected )
            {
                $tab->addStyle('display', 'block');
                $MIOLO->page->onload("if ( !document.mtabbedbasegroup_lastTab ) { miolo.getElementById('{$tab->id}Button').onclick(); }");
            }
            else
            {
                $MIOLO->page->onload("mtabbedbasegroup.changeToLastTab(document.mtabbedbasegroup_lastTab, '$tabbedBaseGroupId');");
            }
        }
        else
        {
            $tempDiv->setClass('mTab mTabDisabled');

            if ( $tab->ajaxAction )
            {
                $url = $MIOLO->getUI()->getAjax($tab->ajaxAction);
                $tempDiv->addAttribute('onclick', "return false;$url");
            }
        }

        return $tempDiv;
    }

    /**
     * Generates the tabs container
     *
     * @return object MHContainer
     */
    public function generate()
    {
        $MIOLO = MIOLO::getInstance();

        $jsCode = "{$this->name}Tabs = new Array(); \n";
        $index = 0;

        $tabButtons = array();

        foreach ( $this->tabs as $tab )
        {
            $tabButtons[$tab->id] = $this->getTabButton($tab, $this->name, $index == 0 ? true : false);

            $this->addControl($tab);
            $jsCode .= "{$this->name}Tabs[{$index}] = '{$tab->id}';\n";

            $index++;
        }

        $divButtons = new MDiv("buttons{$this->name}", $tabButtons);
        $divButtons->setClass('mTabButtons');

        // Shows only the current tab
        $MIOLO->page->addJsCode($jsCode);

        $buttonContainer = new MHContainer("tabButtonContainer{$this->name}", array($divButtons));

        return $buttonContainer->generate() . parent::generate();
    }

    /* Static methods to use on ajax requests */

    /**
     * Enables tab
     *
     * @param string $tabId
     * @param string $tabbedBaseGroupId
     */
    public static function enableTab($tabId, $tabbedBaseGroupId)
    {
        $jsCode = "mtabbedbasegroup.enableTab('{$tabId}', '{$tabbedBaseGroupId}');";

        $MIOLO = MIOLO::getInstance();
        $MIOLO->page->onload($jsCode);
        $MIOLO->ajax->setResponse(NULL, "{$tabbedBaseGroupId}DivResponse");
    }

    /**
     * Disables tab
     *
     * @param string $tabId
     * @param string $tabbedBaseGroupId
     */
    public static function disableTab($tabId, $tabbedBaseGroupId)
    {
        $jsCode = "mtabbedbasegroup.disableTab('{$tabId}', '{$tabbedBaseGroupId}');";

        $MIOLO = MIOLO::getInstance();
        $MIOLO->page->onload($jsCode);
        $MIOLO->ajax->setResponse(NULL, "{$tabbedBaseGroupId}DivResponse");
    }

    /**
     * Removes a tab from the tab list
     *
     * @param string $tabId
     * @param string $tabbedBaseGroupId
     */
    public static function removeTab($tabId, $tabbedBaseGroupId)
    {
        $MIOLO = MIOLO::getInstance();
        $MIOLO->page->onload("mtabbedbasegroup.removeTab('{$tabId}','{$tabbedBaseGroupId}');");
        $MIOLO->ajax->setResponse(NULL, "{$tabbedBaseGroupId}DivResponse");
    }

    /**
     * Updates tab content through ajax
     *
     * @param string $tabId Id of the tab to put content
     * @param array $controls Fields to put in the tab
     */
    public static function updateTab($tabId, $controls)
    {
        $MIOLO = MIOLO::getInstance();
        $container = new MVContainer("ajaxFields$tabId", $controls, MFormControl::FORM_MODE_SHOW_SIDE);

        $MIOLO->ajax->setResponse( $container, $tabId );
    }
}

/**
 * Tab class used in the 
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2010/07/20
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2010 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */
class MTab extends MVContainer
{
    public $label;
    public $ajaxAction;
    public $disabled;

    /**
     * MTab constructor
     *
     * @param string $id Id of the tab
     * @param string $label Label of the tab
     * @param array $controls Fields of the tab
     * @param string $ajaxAction An ajax action to be called when the tab is selected
     * @param boolean $disabled Informs whether the tab is disabled or not
     */
    function __construct($id, $label, $controls, $ajaxAction = NULL, $disabled = false)
    {
        parent::__construct($id, $controls, MFormControl::FORM_MODE_SHOW_SIDE);
        $this->label = $label;
        $this->ajaxAction = $ajaxAction;
        $this->disabled = $disabled;
    }

    /**
     * @param boolean Sets disabled to false
     */
    public function setEnabled()
    {
        $this->disabled = false;
    }

    /**
     * @param boolean Sets disabled to true
     */
    public function setDisabled()
    {
        $this->disabled = true;
    }
}
?>
