<?php

/**
 * Timestamp field with date and time components
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2010/07/26
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2010 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */

class MTimestampField extends MTextField
{
    private $dateField;
    private $timeField;

    public function __construct($name = '', $value = '', $label = '', $readonly = false)
    {
        $MIOLO = MIOLO::getInstance();
        $MIOLO->uses('ui/controls/mtimefield.class.php');

        $this->dateField = new MCalendarField($name . 'Date', NULL, NULL);
        $this->timeField = new MTimeField($name . 'Time');

        $refreshEvent = "miolo.getElementById('{$name}').value = miolo.getElementById('{$name}Date').value + ' ' + miolo.getElementById('{$name}Time').value.replace('T', '');";
        $refreshEvent .= "miolo.getElementById('{$name}Time').value = miolo.getElementById('{$name}Time').value.replace('T', '');";

        $this->dateField->addAttribute('onchange', $refreshEvent);
        $this->timeField->addAttribute('onchange', $refreshEvent);

        parent::__construct($name, NULL, $label);
        $this->setReadOnly($readonly);
        $this->setValue($value);
        $this->addStyle('display', 'none');
    }

    public function getDateField()
    {
        return $this->dateField;
    }

    public function setDateField($dateField)
    {
        $this->dateField = $dateField;
    }

    public function getTimeField()
    {
        return $this->timeField;
    }

    public function setTimeField($timeField)
    {
        $this->timeField = $timeField;
    }

    public function setValue($value)
    {
        list($data, $time) = explode(' ', $value);
        $this->dateField->setValue($data);
        $this->timeField->setValue($time);
        $this->value = $value;
    }

    public function setReadOnly($readonly)
    {
        $this->dateField->setReadOnly($readonly);
        $this->timeField->setReadOnly($readonly);
        $this->readonly = $readonly;
    }

    public function addEvent($event, $action)
    {
        $this->dateField->addEvent($event, $action);
        $this->timeField->addEvent($event, $action);
    }

    public function generateInner()
    {
        parent::generateInner();
        $this->dateField->generateInner();
        $this->timeField->generateInner();
        $this->timeField->generateEvent();
        $this->inner = $this->dateField . $this->timeField->inner . $this->inner;
    }
}
?>
