<?php

class MJasperReport extends MReport
{
    public $filetype; // pdf doc xls rtf htm rpt
    public $fileout;
    public $fileexp;
    public $objDb;
    public $db;
    public $connectionType;

    public function __construct($db = 'admin', $connectionType = 'local')
    {
        $this->db = $db;
        $this->connectionType=$connectionType;
    }

    public function execute($module, $name, $parameters = null, $filetype = 'PDF', $save = false)
    {
        $MIOLO = MIOLO::getInstance();
        $page = $MIOLO->getPage();

        $this->filetype = isset($filetype) ? $filetype : 'PDF';

        $filein = addslashes($MIOLO->GetModulePath($module, 'reports/' . $name . '.jasper'));

        $uniqid = uniqid(md5(uniqid("")));
        $this->fileout= $uniqid . "." . strtolower($this->filetype);
        $pathout = $MIOLO->getConf("home.reports") .'/'.$this->fileout;

        $comando = 'echo $JAVA_HOME';
        $pathJava = trim(shell_exec($comando));

        $pathMJasper = $MIOLO->getConf("home.extensions")."/jasper";
        $pathScriptlets = $MIOLO->GetModulePath($module, 'reports/scriptlets.jar');
        $classPath = "$pathMJasper/lib/*:$pathMJasper/:$pathScriptlets:$pathJava/lib";

        return $this->fill($filein, $pathout, $filetype, $parameters, $classPath, $save);
    }
     
    public function fill($fileIn, $fileOut, $filetype, $parameters, $classPath, $save)
    {
        $MIOLO = MIOLO::getInstance();
        $page = $MIOLO->getPage();
         
        if ($this->connectionType=='local')
        {
            //Solution without TomCat
            $param = "";
            if (is_array($parameters))
            {
                foreach ($parameters as $pn => $pv)
                {
                    $param .= '&' . $pn . "~" . $pv;
                }
            }

            $this->objDb = $MIOLO->GetDatabase($this->db);
            $dbUser = $this->objDb->user;
            $dbPass = $this->objDb->pass;
            $dbHost = $this->objDb->host;
            $dbName = $this->objDb->db;
            $dbPort = $this->objDb->port;
            
            if ( $this->objDb->system == 'postgres' )
            {
                $jdbcDriver = 'org.postgresql.Driver';
                $dbPort = $dbPort ? $dbPort : '5432';
                $jdbcDb = "jdbc:postgresql://{$dbHost}:{$dbPort}/{$dbName}";
            }
            elseif ( $this->objDb->system == 'mysql' )
            {
                $jdbcDriver = 'com.mysql.jdbc.Driver';
                $dbPort = $dbPort ? $dbPort : '3306';
                $jdbcDb = "jdbc:mysql://{$dbHost}:{$dbPort}/{$dbName}";
            }
            else
            {
                $jdbcDriver = 'org.postgresql.Driver';
                $jdbcDb = "jdbc:postgresql://localhost:5432/postgres";
                $dbUser = "postgres";
                $dbPass = "postgres";
            }

            $param = "relatorio~$fileIn".$param."&fileout~".$fileOut."&filetype~".$this->filetype;

            $comando = 'echo $JAVA_HOME';
            $pathJava = trim(shell_exec($comando));
            $pathLog = $MIOLO->GetConf('home.logs');
            $uniqid = uniqid(md5(uniqid("")));
            $fileLog = $uniqid . ".txt";

            $pathJava = $pathJava ? $pathJava . '/bin/java' : 'java'; //if JAVA_HOME is unsetted, get java from default path

            $lang = $MIOLO->getConf('i18n.language');
            $cmd = "export LANG=$lang; " . $pathJava  . " -classpath $classPath MJasper \"{$pathMJasper}\" \"{$param}\" \"{$dbUser}\" \"{$dbPass}\" \"{$jdbcDriver}\" \"{$jdbcDb}\"";
            $cmd .= " 2> $pathLog/$fileLog";

            exec($cmd, $output);

            // Remove the log file if it's empty
            $log = fopen("$pathLog/$fileLog", 'r');
            $logData = fread($log, filesize("$pathLog/$fileLog"));
            fclose($log);
            if ( $logData == '' )
            {
                unlink("$pathLog/$fileLog");
            }

            //JAVA application without erros!
            if (trim($output[0])=="end")
            {
                //Break line incompatibility problem with Windows and Unix
                if (strtoupper(trim($this->filetype))=="TXT") Mutil::unix2dos($fileOut);

                if ($save) {
                    $MIOLO->response->sendDownload($fileOut);
                } else {
                    $this->fileout = $MIOLO->getActionURL('miolo','reports:'.$this->fileout);
                    $MIOLO->getPage()->window($this->fileout);
                }
                return 1;
            }
            elseif ( trim($output[0]) == 'empty' )
            {
                return 0;
            }
            else
            {
                $message = implode('<br>', $output) != 'null' ? implode('<br>', $output) : _M('The report could not be generated');
                $detail = "\n$cmd\n$logData";
                $div = new MExpandDiv('errorDiv', $message . $detail);

                throw new EControlException($div->generate());
            }
        }
        else if ($this->connectionType=='remote')
        {
            //TomCat
            $this->fileout = $MIOLO->getConf("home.url_jasper"). "?bd={$this->db}&relatorio=$filein" . $param;
            $MIOLO->getPage()->window($this->fileout);
        }
    }
     
    public function executeJRXML($fileIn, $parameters = null, $filetype = 'PDF', $save = false)
    {
        $MIOLO = MIOLO::getInstance();
         
        $this->filetype = isset($filetype) ? $filetype : 'PDF';

        $uniqid = uniqid(md5(uniqid("")));
        $this->fileout = $uniqid . "." . strtolower($this->filetype);
        $pathout = $MIOLO->getConf("home.reports") .'/'.$this->fileout;

        $pathMJasper = $MIOLO->getConf("home.extensions")."/jasper";
        $classPath = "$pathMJasper/lib/*:$pathMJasper/";

        $this->fill($fileIn, $pathout, $this->filetype, $parameters, $classPath, $save);
    }
}
?>
