<?php

/**
 * Miolo code generator
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 * @author Fernando Kochhann [fernando@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2011/02/09
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Solues Livres \n
 *
 * \b Copyright: \n
 * Copyright (c) 2011 SOLIS - Cooperativa de Solues Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */
class MIOLOGenerate
{
    public $gModule;
    public $dbConf;
    public $table;
    public $template;
    public $vars;
    public $dbinfo;
    public $lookup;
    public $columns;
    public $attributes;
    public $attributesPK;
    public $attributesFK;
    public $pkeys;
    public $fkeys;
    public $substituteIds;
    public $author;
    public $vModule;
    private $className;

    public function __construct($dbConf=NULL, $module=NULL, $schema=NULL, $table=NULL, $system=NULL, $host=NULL, $port=NULL, $name=NULL, $user=NULL, $password=NULL)
    {
        $MIOLO = MIOLO::getInstance();
        $this->gModule = $module;
        $this->dbConf = $dbConf;
        $this->table = $table;
        $this->lookup = false;
        $this->className = self::getClassName($table);

        if ( $dbConf || ( $table && $system && $host && $port && $name && $user ) )
        {
            $MIOLO->uses('classes/DBInfo.class.php', MIOLO::getCurrentModule());
            $this->dbinfo = new DBInfo($dbConf, NULL, $system, $host, $port, $name, $user, $password);
            $this->dbinfo->setSchema($schema);

            $this->columns = $this->dbinfo->getColumns($this->table);
            $this->pkeys = $this->dbinfo->getPKeys($this->table);
            $this->fkeys = $this->dbinfo->getFKeys($this->table);

            foreach ( $this->columns as $column )
            {
                $attribute = $column[0];

                if ( strpos($attribute, '_') == 4 )
                {
                    $attribute = substr($attribute, 5);
                }

                $this->attributes[] = self::generateLowerCamelCaseName($attribute);
            }

            foreach ( $this->pkeys as $pkey )
            {
                $attributePK = $pkey[0];

                if ( strpos($attributePK, '_') == 4 )
                {
                    $attributePK = substr($attributePK, 5);
                }

                $this->attributesPK[] = self::generateLowerCamelCaseName($attributePK);
            }

            foreach ( $this->fkeys as $fkey )
            {
                $attributesFK = array( );
                foreach ( $fkey as $index => $fkeyinfo )
                {
                    $attFK = $fkeyinfo;

                    if ( $index == 3 ) // from_column_name
                    {
                        $attFK = substr($attFK, 5);
                    }

                    $attributesFK[] = self::generateLowerCamelCaseName($attFK);
                }
                $this->attributesFK[] = $attributesFK;
            }
        }
    }

    public function generateClass()
    {
        if ( $this->substituteIds )
        {
            foreach  ( $this->attributes as $key => $attribute )
            {
                if ( $this->substituteIds[$attribute] )
                {
                    $this->attributes[$key] = $this->substituteIds[$attribute];
                }
            }
            foreach  ( $this->attributesPK as $key => $attribute )
            {
                if ( $this->substituteIds[$attribute] )
                {
                    $this->attributesPK[$key] = $this->substituteIds[$attribute];
                }
            }
            foreach  ( $this->attributesFK as $key => $attribute )
            {
                if ( $this->substituteIds[$attribute] )
                {
                    $this->attributesFK[$key] = $this->substituteIds[$attribute];
                }
            }
        }


        $empty = $this->findVars();

        foreach ( $empty as $var )
        {
            unset($str);
            unset($i);
            if ( $this->vars[$var] )
            {
                $this->generated = str_replace('%' . $var . '%', $this->vars[$var], $this->generated);
            }
            else
            {
                switch ( $var )
                {
                    case 'CURRENT_DATE':
                        $this->generated = str_replace('%CURRENT_DATE%', date('d/m/Y'), $this->generated);
                        break;

                    case 'MODULE':
                        $this->generated = str_replace('%MODULE%', $this->vModule, $this->generated);
                        break;

                    case 'AUTHOR':
                        $this->generated = str_replace('%AUTHOR%', $this->author, $this->generated);
                        break;

                    case 'FORM_NAME':
                        $this->generated = str_replace('%FORM_NAME%', $this->className, $this->generated);
                        break;

                    case 'FORM_LABEL':
                        $this->generated = str_replace('%FORM_LABEL%', $this->className, $this->generated);
                        break;

                    case 'FORM_FIELDS':
                        $this->generated = str_replace('%FORM_FIELDS%', $this->generateFormFields(), $this->generated);
                        break;

                    case 'FORM_VALIDATORS':
                        $this->generated = str_replace('%FORM_VALIDATORS%', $this->generateFormValidators(), $this->generated);
                        break;

                    case 'FORM_FILTERS_SAGU':
                        $this->generated = str_replace('%FORM_FILTERS_SAGU%', $this->generateFormFiltersSAGU(), $this->generated);
                        break;

                    case 'FORM_FILTERS_VALIDATORS':
                        $this->generated = str_replace('%FORM_FILTERS_VALIDATORS%', $this->generateFormValidators(true), $this->generated);
                        break;

                    case 'FORM_PRIMARY_KEYS':
                        $pkeys = '';
                        foreach ( $this->attributesPK as $pk )
                        {
                            $pkeys .= ", '$pk'";
                        }
                        $pkeys = substr($pkeys, 2);
                        $this->generated = str_replace('%FORM_PRIMARY_KEYS%', $pkeys, $this->generated);
                        break;

                    case 'FORM_PRIMARY_KEYS_VARS':
                        $pkeys = '';
                        foreach ( $this->attributesPK as $pk )
                        {
                            $pkeys .= ", \$$pk";
                        }
                        $pkeys = substr($pkeys, 2);
                        $this->generated = str_replace('%FORM_PRIMARY_KEYS_VARS%', $pkeys, $this->generated);
                        break;

                    case 'FORM_PRIMARY_KEYS_HIDDEN':
                        $this->generated = str_replace('%FORM_PRIMARY_KEYS_HIDDEN%', $this->generateHiddenPKFields(), $this->generated);
                        break;

                    case 'GRID_COLUMNS':
                        $this->generated = str_replace('%GRID_COLUMNS%', $this->generateGridColumns(), $this->generated);
                        break;

                    case 'TABLE':
                        $this->generated = str_replace('%TABLE%', $this->table, $this->generated);
                        break;

                    case 'TABLE_COLUMNS':
                        $this->generated = str_replace('%TABLE_COLUMNS%', $this->generateTableColumns(), $this->generated);
                        break;

                    case 'TABLE_INSERT_COLUMNS':
                        $this->generated = str_replace('%TABLE_INSERT_COLUMNS%', $this->generateTableInsertColumns(), $this->generated);
                        break;

                    case 'TABLE_PRIMARY_KEYS':
                        $this->generated = str_replace('%TABLE_PRIMARY_KEYS%', $this->generateTablePrimaryKeys(), $this->generated);
                        break;

                    case 'TABLE_CLASS_NAME':
                        $this->generated = str_replace('%TABLE_CLASS_NAME%', $this->className, $this->generated);
                        break;

                    case 'WHERE_PRIMARY_KEYS':
                        $this->generated = str_replace('%WHERE_PRIMARY_KEYS%', $this->generateWherePrimaryKeys(), $this->generated);
                        break;

                    case 'LOOKUP_FIRST_FIELD':
                        $this->generated = str_replace('%LOOKUP_FIRST_FIELD%', $this->attributes[0], $this->generated);
                        break;

                    case 'LOOKUP_FILTERS':
                        $this->generated = str_replace('%LOOKUP_FILTERS%', $this->generateLookupFilters(), $this->generated);
                        break;

                    case 'LOOKUP_COLUMNS':
                        $this->generated = str_replace('%LOOKUP_COLUMNS%', $this->generateLookupColumns(), $this->generated);
                        break;

                    case 'LOOKUP_SQL_FILTERS':
                        $this->generated = str_replace('%LOOKUP_SQL_FILTERS%', $this->generateSqlFilters(true), $this->generated);
                        break;

                    case 'SQL_FILTERS':
                        $this->generated = str_replace('%SQL_FILTERS%', $this->generateSqlFilters(), $this->generated);
                        break;

                    case 'TYPE_ATTRIBUTES':
                        $this->generated = str_replace('%TYPE_ATTRIBUTES%', $this->generateTypeAttributes(), $this->generated);
                        break;

                    case 'TYPE_CHECK_PRIMARY_KEYS':
                        $this->generated = str_replace('%TYPE_CHECK_PRIMARY_KEYS%', $this->generateTypeCheckPrimaryKeys(), $this->generated);
                        break;

                    case 'TYPE_CLEAN_PRIMARY_KEYS':
                        $this->generated = str_replace('%TYPE_CLEAN_PRIMARY_KEYS%', $this->generateCleanPrimaryKeys(), $this->generated);
                        break;

                    case 'TYPE_CONSTRUCT_PARAMETERS':
                        $this->generated = str_replace('%TYPE_CONSTRUCT_PARAMETERS%', $this->generateTypeConstructParameters(), $this->generated);
                        break;

                    case 'TYPE_CONSTRUCT_CHECK_PKEYS':
                        $this->generated = str_replace('%TYPE_CONSTRUCT_CHECK_PKEYS%', $this->generateTypeConstructCheckPKeys(), $this->generated);
                        break;

                    case 'TYPE_CONSTRUCT_INIT_PKEYS':
                        $this->generated = str_replace('%TYPE_CONSTRUCT_INIT_PKEYS%', $this->generateTypeConstructInitPKeys(), $this->generated);
                        break;

                    case 'TYPE_INSERT_VALUES':
                        $this->generated = str_replace('%TYPE_INSERT_VALUES%', $this->generateInsertValues(), $this->generated);
                        break;

                    case 'TYPE_PARAMETERS':
                        $this->generated = str_replace('%TYPE_PARAMETERS%', $this->generateTypeParameters(), $this->generated);
                        break;

                    case 'TYPE_INSERT_ARGS':
                        $this->generated = str_replace('%TYPE_INSERT_ARGS%', $this->generateInsertArgs(), $this->generated);
                        break;

                    case 'TYPE_INSERT_INITIALIZE':
                        $this->generated = str_replace('%TYPE_INSERT_INITIALIZE%', $this->generateInsertInitialize(), $this->generated);
                        break;

                    case 'TYPE_UPDATE_SET':
                        $this->generated = str_replace('%TYPE_UPDATE_SET%', $this->generateUpdateSet(), $this->generated);
                        break;

                    case 'TYPE_UPDATE_PARAMETERS':
                        $parameters = array();

                        foreach ( $this->attributes as $att )
                        {
                            if ( !in_array($att, $this->attributesPK) )
                            {
                                $parameters[] = "\$this->$att";
                            }
                        }

                        foreach ( $this->attributesPK as $att )
                        {
                            $parameters[] = "\$this->$att";
                        }
                        $this->generated = str_replace('%TYPE_UPDATE_PARAMETERS%', implode(",\n        ", $parameters), $this->generated);
                        break;

                    case 'TYPE_PRIMARY_KEYS':
                        $this->generated = str_replace('%TYPE_PRIMARY_KEYS%', $this->generateTypePrimaryKeys(), $this->generated);
                        break;
                }
            }
        }

        return $this->generated;
    }

    private function generateFormFields()
    {
        foreach ( $this->columns as $column )
        {
            $type = $column[1];
            $columnName = $column[0];
            $name = $this->substituteIds[$columnName];
            $value = '';
            $label = $columnName;
            $lookup = false;
            $maxlength = $column[5];

            if ( $this->fkeys )
            {
                foreach ( $this->fkeys as $fk )
                {
                    if ( $fk[3] == $columnName )
                    {
                        $lookup = true;
                        $fields .= $this->generateLookupTextField($name, $value, $label);
                        $fields .= "\$fields[] = \$$name;\n";
                    }
                }
            }
            if ( !$lookup )
            {
                $fields .= $this->generateFieldByType($name, $value, $label, $type, $maxlength);
            }
        }
        return $fields;
    }

    private function generateFormValidators($search=false)
    {
        $validators = '';
        foreach ( $this->columns as $column )
        {
            $type = $column[1];
            $columnName = $column[0];
            $name = $this->substituteIds[$columnName];
            $value = '';
            $label = $columnName;
            $lookup = false;

            // search form does not have required validators
            if ( !$search )
            {
                $required = $column[3] == 't' ? 'required' : 'optional';
            }
            else
            {
                $name = $name.'S';
                $required = 'optional';
            }

            switch ( $type )
            {
                case 'integer':
                    $validators .= "\$validators[] = new MIntegerValidator('$name', '', '$required');\n";
                    break;

                case 'text':
                case 'date':
                case 'boolean':
                case 'double precision':
                case 'real':
                case 'numeric':
                case 'time without time zone':
                case 'timestamp without time zone':
                case 'character':
                case 'character varying':
                default:
                    if ( $required == 'required' )
                    {
                        $validators .= "\$validators[] = new MRequiredValidator('$name');\n";
                    }
                    break;
            }
        }

        return $validators;
    }

    private function generateFieldByType($name, $value, $label, $type, $maxlength, $search=false)
    {
        $fields = '';
        $size = null;
        switch ( $type )
        {
            case 'text':
                if ( $maxlength )
                {
                    $fields .= "\$$name = new MTextArea('$name', '$value', _M('$label', \$module));\n";
                    $fields .= "\${$name}->addAttribute('maxlength', '$maxlength');\n";
                    $fields .= "\$fields[] = \$$name;\n";
                }
                else
                {
                    $fields .= "\$fields[] = new MTextArea('$name', '$value', _M('$label', \$module));\n";
                }

                break;

            case 'date':
                $size = 10;
                $fields .= "\$fields[] = new MCalendarField('$name', '$value', _M('$label', \$module), $size);\n";
                break;

            case 'boolean':
                if ( $search )
                {
                    $fields .= "\$fields[] = new MSelection('$name', '$value', _M('$label', \$module), SAGU::listYesNo(0));\n";
                }
                else
                {
                    $fields .= "\$fields[] = new MRadioButtonGroup('$name', _M('$label', \$module), SAGU::listYesNo(1), '$value', NULL, 'horizontal');";
                }
                break;

            case 'integer':
            case 'double precision':
            case 'real':
            case 'numeric':
            case 'time without time zone':
                $size = $size ? $size : 10;
            case 'timestamp without time zone':
            case 'character':
            case 'character varying':
            default:
                $size = $size ? $size : 20;
                if ( $maxlength )
                {
                    $fields .= "\$$name = new MTextField('$name', '$value', _M('$label', \$module), $size);\n";
                    $fields .= "\${$name}->addAttribute('maxlength', '$maxlength');\n";
                    $fields .= "\$fields[] = \$$name;\n";
                }
                else
                {
                    $fields .= "\$fields[] = new MTextField('$name', '$value', _M('$label', \$module), $size);\n";
                }
                break;
        }

        return $fields;
    }

    private function generateHiddenPKFields()
    {
        $fields = '';

        // separates vars initializing from fields creation
        foreach ( $this->attributesPK as $pk )
        {
            $fields .= "\$$pk = MIOLO::_REQUEST('$pk');\n";
        }

        foreach ( $this->attributesPK as $pk )
        {
            $fields .= "\$fields[] = new MHiddenField('$pk', \$$pk);\n";
        }

        return $fields;
    }

    public function generateGridColumns()
    {
        $i = 0;
        $columns = '';
        $pks = '$primaryKeys = array( ';

        foreach ( $this->columns as $column )
        {
            $attribute = $this->substituteIds[$column[0]];

            switch ( $column[1] )
            {
                case 'integer':
                    $align = 'right';
                    break;

                case 'boolean':
                    $align = 'center';
                    break;

                default:
                    $align = 'left';
                    break;
            }

            $columns .= "\$columns[] = new MGridColumn( _M('$attribute', \$module), '$align', true, NULL, true, NULL, true );\n";

            if ( in_array($attribute, $this->attributesPK) )
            {
                $pks .= "'$attribute' => '%$i%',\n";
            }

            $i++;
        }

        $pks .= " );\n";

        return $columns . "\n" . $pks;
    }

    private function generateTableColumns()
    {
        $columns = array();

        foreach ( $this->columns as $column )
        {
            switch ( $column[1] )
            {
                case 'date':
                    $columns[] = "TO_CHAR($column[0], \'' . SAGU::getParameter('BASIC', 'MASK_DATE') . '\')";
                    break;

                case 'timestamp with time zone':
                case 'timestamp without time zone':
                    $columns[] = "TO_CHAR($column[0], \'' . SAGU::getParameter('BASIC', 'MASK_TIMESTAMP') . '\')";
                    break;

                case 'time':
                    $columns[] = "TO_CHAR($column[0], \'' . SAGU::getParameter('BASIC', 'MASK_TIME') . '\')";
                    break;

                default:
                    $columns[] = $column[0];
                    break;
            }
        }

        // spaces to let the code less ugly
        return implode(",\n                       ", $columns);
    }

    private function generateTableInsertColumns()
    {
        $columns = array();

        foreach ( $this->columns as $column )
        {
            $columns[] = $column[0];
        }

        return implode(', ', $columns);
    }

    private function generateWherePrimaryKeys()
    {
        $pkeys = array();

        foreach ( $this->pkeys as $column )
        {
            $pkeys[] = $column[0];
        }

        return implode(' = ? AND ', $pkeys) . ' = ?';
    }

    private function generateLookupFilters()
    {
        $fields = '';

        foreach ( $this->columns as $column )
        {
            $type = $column[1];
            $columnName = $column[0];
            $name = $this->substituteIds[$columnName];
            $value = '';
            $label = $columnName;
            $lookup = false;
            $maxlength = $column[5];

            if ( $this->fkeys )
            {
                foreach ( $this->fkeys as $fk )
                {
                    if ( $fk[3] == $columnName )
                    {
                        $lookup = true;
                        $fields .= $this->generateLookupTextField($name, $value, $label);
                        $fields .= "\$lookup->addFilterField(\$$name);\n";
                    }
                }
            }
            if ( !$lookup )
            {
                $size = null;
                switch ( $type )
                {
                    case 'text':
                        if ( $maxlength )
                        {
                            $fields .= "\$$name = new MTextArea('$name', '$value', _M('$label', \$module));\n";
                            $fields .= "\${$name}->addAttribute('maxlength', '$maxlength');\n";
                            $fields .= "\$lookup->addFilterField(\$$name);\n";
                        }
                        else
                        {
                            $fields .= "\$lookup->addFilterField(new MTextArea('$name', '$value', _M('$label', \$module)));\n";
                        }

                        break;

                    case 'date':
                        $size = 10;
                        $fields .= "\$lookup->addFilterField(new MCalendarField('$name', '$value', _M('$label', \$module), $size));\n";
                        break;

                    case 'boolean':
                        if ( $search )
                        {
                            $fields .= "\$lookup->addFilterField(new MSelection('$name', '$value', _M('$label', \$module), SAGU::listYesNo(0)));\n";
                        }
                        else
                        {
                            $fields .= "\$lookup->addFilterField(new MRadioButtonGroup('$name', _M('$label', \$module), SAGU::listYesNo(1), '$value', NULL, 'horizontal'));";
                        }
                        break;

                    case 'integer':
                    case 'double precision':
                    case 'real':
                    case 'numeric':
                    case 'time without time zone':
                        $size = $size ? $size : 10;
                    case 'timestamp without time zone':
                    case 'character':
                    case 'character varying':
                    default:
                        $size = $size ? $size : 20;
                        if ( $maxlength )
                        {
                            $fields .= "\$$name = new MTextField('$name', '$value', _M('$label', \$module), $size);\n";
                            $fields .= "\${$name}->addAttribute('maxlength', '$maxlength');\n";
                            $fields .= "\$lookup->addFilterField(\$$name);\n";
                        }
                        else
                        {
                            $fields .= "\$lookup->addFilterField(new MTextField('$name', '$value', _M('$label', \$module), $size));\n";
                        }
                        break;
                }
            }
        }

        return $fields;
    }

    private function generateLookupColumns()
    {
        $columns = '';

        foreach ( $this->attributes as $attribute )
        {
            $columns .= "\$columns[] = new MDataGridColumn('$attribute', _M('$attribute', \$module), 'left', true, NULL, true);\n";
        }

        return $columns;
    }

    private function generateSqlFilters($lookup=false)
    {
        $filters = '';

        foreach ( $this->columns as $column )
        {
            $name = $this->substituteIds[$column[0]];
            $where = '';
            $percent = '';

            switch ( $column[1] )
            {
                case 'date':
                    $where = "$column[0] = TO_DATE(?, \'' . SAGU::getParameter('BASIC', 'MASK_DATE') . '\')";
                    break;

                case 'timestamp with time zone':
                case 'timestamp without time zone':
                    $where = "$column[0] = TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIMESTAMP') . '\')";
                    break;

                case 'time':
                    $where = "$column[0] = TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIME') . '\')";
                    break;

                case 'text':
                case 'character varying':
                case 'varchar':
                    $where = "TO_ASCII($column[0]) ILIKE TO_ASCII(?) ";
                    $percent = " . '%'";
                    break;

                default:
                    $where = "$column[0] = ?";
                    break;
            }

            if ( !$lookup )
            {
                $name = "filters->$name";
            }

            $filters .= "
            if ( strlen(\$$name) > 0 )
            {
                \$where .= ' AND $where';
                \$args[] = \$$name$percent;
            }";
        }

        return $filters;
    }

    private function generateUpdateSet()
    {
        $update = array();

        foreach ( $this->columns as $column )
        {
            $set = '';
            $name = $this->substituteIds[$column[0]];

            if ( in_array($name, $this->attributesPK) )
            {
                continue;
            }

            switch ( $column[1] )
            {
                case 'date':
                    $set = "$column[0] = TO_DATE(?, \'' . SAGU::getParameter('BASIC', 'MASK_DATE') . '\')";
                    break;

                case 'timestamp with time zone':
                case 'timestamp without time zone':
                    $set = "$column[0] = TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIMESTAMP') . '\')";
                    break;

                case 'time':
                    $set = "$column[0] = TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIME') . '\')";
                    break;

                default:
                    $set = "$column[0] = ?";
                    break;
            }

            $update[] = $set;
        }

        return implode(",\n                       ", $update);
    }

    private function generateInsertValues()
    {
        $insert = array();

        foreach ( $this->columns as $column )
        {
            $name = $this->substituteIds[$column[0]];
            $value = '?';

            switch ( $column[1] )
            {
                case 'date':
                    $value = "TO_DATE(?, \'' . SAGU::getParameter('BASIC', 'MASK_DATE') . '\')";
                    break;

                case 'timestamp with time zone':
                case 'timestamp without time zone':
                    $value = "TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIMESTAMP') . '\')";
                    break;

                case 'time':
                    $value = "TO_TIMESTAMP(?, \'' . SAGU::getParameter('BASIC', 'MASK_TIME') . '\')";
                    break;
            }

            $insert[] = $value;
        }

        return implode(', ', $insert);
    }

    private function generateTablePrimaryKeys()
    {
        $pkeys = array();

        foreach ( $this->pkeys as $column )
        {
            $pkeys[] = $column[0];
        }

        return implode(', ', $pkeys);
    }

    private function generateTypeAttributes()
    {
        $atts = '';

        foreach ( $this->columns as $column )
        {
            $name = $this->substituteIds[$column[0]];
            $type = $column[1];
            $description = $column[2];

            $atts .= "/**\n";
            $atts .= " * @AttributeType $type\n";
            $atts .= " * $description\n";
            $atts .= " */\n";
            $atts .= "protected \$$name;";
        }

        return $atts;
    }

    private function generateTypeCheckPrimaryKeys()
    {
        $check = array();

        foreach ( $this->attributesPK as $pk )
        {
            $check[] = "strlen(\$this->$pk) == 0";
        }

        return implode(' || ', $check);
    }

    private function generateTypeConstructCheckPKeys()
    {
        $check = array();

        foreach ( $this->attributesPK as $pk )
        {
            $check[] = "(strlen(\$$pk) > 0)";
        }

        return implode(' && ', $check);
    }

    private function generateTypeParameters()
    {
        $parameters = array();

        foreach ( $this->attributes as $att )
        {
            $parameters[] = "\$this->$att";
        }

        return implode(', ', $parameters);
    }

    private function generateTypeConstructParameters()
    {
        $parameters = array();

        foreach ( $this->attributesPK as $att )
        {
            $parameters[] = "\$$att=NULL";
        }

        return implode(', ', $parameters);
    }

    private function generateInsertArgs()
    {
        $args = '';
        $names = array();
        $sequential = false;

        foreach ( $this->pkeys as $pk )
        {
            // check if it is sequential
            if ( substr_compare($pk[2], 'nextval', 0) > 0 )
            {
                $name = $this->substituteIds[$pk[0]];
                $sequential = true;

                $args .= "\$sqlPK = \"SELECT $pk[2]\";";
                $args .= '$result = SDatabase::query($sqlPK);';
                $args .= "\$$name = \$result[0][0];";

                $names[] = "\$$name";
            }
        }

        $args .= "\$args = array(\n";
        if ( !$sequential )
        {
            $args .= $this->generateTypeParameters();
        }
        else
        {
            foreach ( $this->attributes as $att )
            {
                if ( !in_array($att, $this->attributesPK) )
                {
                    $names[] = "\$this->$att";
                }
            }
            $args .= implode(",\n    ", $names);
        }
        $args .= ");\n";

        return $args;
    }

    private function generateInsertInitialize()
    {
        $args = '';
        $names = array();
        $sequential = false;

        foreach ( $this->pkeys as $pk )
        {
            // check if it is sequential
            if ( substr_compare($pk[2], 'nextval', 0) > 0 )
            {
                $name = $this->substituteIds[$pk[0]];
                $sequential = true;

                $names[] = "\$this->$name = \$$name;";
            }
        }

        if ( $sequential )
        {
            $args .= "if ( \$result )\n{\n";
            $args .= implode("\n", $names);
            $args .= "}\n";
        }

        return $args;
    }

    private function generateTypeConstructInitPKeys()
    {
        $names = array();

        foreach ( $this->attributesPK as $pk )
        {
            $names[] = "\$this->$pk = \$$pk;";
        }

        return implode("\n", $names);
    }

    private function generateTypePrimaryKeys()
    {
        $pkeys = array();

        foreach ( $this->attributesPK as $pk )
        {
            $pkeys[] = "\$this->$pk";
        }

        return implode(', ', $pkeys);
    }

    private function generateCleanPrimaryKeys()
    {
        $pkeys = array();

        foreach ( $this->attributesPK as $pk )
        {
            $pkeys[] = "\$this->$pk = null;";
        }

        return implode("\n    ", $pkeys);
    }

    /*
     * SAGU
     */
    private function generateFormFiltersSAGU()
    {
        $fields = '';

        foreach ( $this->columns as $column )
        {
            $type = $column[1];
            $columnName = $column[0];
            $name = $this->substituteIds[$columnName];
            $value = '';
            $label = $columnName;
            $lookup = false;
            $maxlength = $column[5];

            if ( $this->fkeys )
            {
                foreach ( $this->fkeys as $fk )
                {
                    if ( $fk[3] == $columnName )
                    {
                        $lookup = true;
                        $fields .= $this->generateLookupTextField($name . 'S', $value, $label);
                        $fields .= "\$fields[] = \${$name}S;\n";
                    }
                }
            }
            if ( !$lookup )
            {
                $fields .= $this->generateFieldByType($name . 'S', $value, $label, $type, $maxlength, true);
            }
        }

        return $fields;
    }

    private function generateLookupTextField($name, $value, $label)
    {
        $lookup = "\${$name}Label = new MText('{$name}Label', _M('$label', \$module) . ':');\n";
        $lookup .= "\${$name}Label->setWidth(SAGU::getParameter('BASIC', 'FIELD_LABEL_SIZE'));\n";
        $lookup .= "\${$name} = new MLookupTextField('{$name}Choose', \$this->getFormValue('{$name}Choose', \$data->{$name}Choose), NULL, SAGU::getParameter('BASIC', 'FIELD_LOOKUPFIELD_SIZE'), NULL, NULL, '{$name}Description', \$module, '$name', NULL, NULL, true);\n";
        $lookup .= "\${$name}->setJsHint(_M('Informe o cÃ³digo do $label', \$module));\n";
        $lookup .= "\${$name}Description = new MTextField('{$name}Description', \$this->getFormValue('{$name}Description', \$data->{$name}Description), NULL, SAGU::getParameter('BASIC', 'FIELD_DESCRIPTION_SIZE'), NULL, true);\n";
        $lookup .= "\${$name}Description->setJsHint(_M('DescriÃ§Ã£o do $label', \$module));\n";
        $lookup .= "\$$name = new MHContainer('{$name}Cnt',array(\${$name}Label, \${$name}, \${$name}Description));\n";
        return $lookup;
    }


    public function setSubstituteIds($columnIds, $formIds)
    {
        foreach ( $columnIds as $key => $columnId )
        {
            $this->substituteIds[$columnId] = $formIds[$key];
        }
    }

    public function setClassName($className)
    {
        $this->className = $className;
    }

    public function getConstArray()
    {
        return self::$const;
    }

    public function setTemplate($template)
    {
        $this->template = $template;
        $handle = fopen($this->template, "r");
        $this->generated = fread($handle, filesize($this->template));
    }

    public function setVars($vars)
    {
        $this->vars = $vars;
    }

    public function findVars()
    {
        if ( preg_match_all("/%([A-Z_]+)%/", $this->generated, $vars) > 0 )
        {
            $otherVars = $vars[1];
            $otherVars = array_unique($otherVars);
            return $otherVars;
        }
        else
        {
            return false;
        }
    }

    public static function generateUpperCamelCaseName($name)
    {
        $camelCased = '';
        foreach ( explode('_', $name) as $piece )
        {
            $camelCased .= ucfirst($piece);
        }
        return $camelCased;
    }

    public static function generateLowerCamelCaseName($name)
    {
        return lcfirst(self::generateUpperCamelCaseName($name));
    }

    public static function getClassName($table)
    {
        return self::generateUpperCamelCaseName($table);
    }

}
?>
