<?php

/**
 * Template upload form
 *
 * TODO: Remove and edit templates
 *
 * @author Daniel Hartmann [daniel@solis.coop.br]
 *
 * @version $id$
 *
 * \b Maintainers: \n
 * Armando Taffarel Neto [taffarel@solis.coop.br]
 * Daniel Hartmann [daniel@solis.coop.br]
 *
 * @since
 * Creation date 2011/02/15
 *
 * \b Organization: \n
 * SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b CopyRight: \n
 * Copyright (c) 2011 SOLIS - Cooperativa de Soluções Livres \n
 *
 * \b License: \n
 * Licensed under GPLv2 (for further details read the COPYING file or http://www.gnu.org/licenses/gpl.html)
 *
 * \b History: \n
 * See history in CVS repository: http://www.miolo.org.br
 *
 */

$MIOLO->uses('forms/frmGenerate.class.php', $module);

class frmTemplate extends MForm
{
    public function __construct()
    {
        $module = MIOLO::getCurrentModule();
        parent::__construct(_M('Template', $module));
        if ( $this->page->isPostBack() )
        {
            $this->eventHandler();
        }
    }

    public function createFields()
    {
        $MIOLO = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();

        $fields[] = MMessage::getMessageContainer();
        $fields[] = MPopup::getPopupContainer();

        $template[] = $this->getTemplateList();
        $fields[] = new MFormContainer('cTemplate', $template, 'vertical', MFormControl::FORM_MODE_SHOW_SIDE);

        $fields[] = new MFileField('templateFile', '', _M('Upload template', $module));

        $this->setFields($fields);
    }

    public function getTemplateList()
    {
        $templates = frmGenerate::getTemplates();
        $template = new MMultiSelection('template', NULL, _M('Template list', $module), $templates);
        $template->addAttribute('disabled', 'disabled');
        $template->addStyle('width', '215px');

        return $template;
    }

    public function editPopup()
    {
        MPopup::show(NULL, $content, _M('Edit template', $module));
    }

    public function confirmOverwrite()
    {
        $module = MIOLO::getCurrentModule();
        $args = MUtil::getAjaxActionArgs();

        if ( !copy($args->tmpFile, $args->path) )
        {
            new MMessageWarning(_M('The file could not be uploaded', $module));
        }

        unlink($args->tmpFile);

        $control = $this->getTemplateList();
        MPopup::remove();
        $this->setResponse($control, 'cTemplate', true);

        new MMessageSuccess(_M('File @1 uploaded successfully', $module, $args->name));
    }

    public function uploadFiles($path)
    {
        $MIOLO = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();
        $uploadInfo = MFileField::getUploadInfo();

        if ( !is_array($uploadInfo) )
        {
            return;
        }

        $uploadedFiles = array();

        foreach ( $uploadInfo as $fileInfo )
        {
            list($fileName, $tmpFile) = explode(';', $fileInfo);
            $tmpFile = $MIOLO->getConf('home.html') . "/files/tmp/$tmpFile";

            if ( file_exists("$path/$fileName") )
            {
                $message = _M('The template already exists, do you want to overwrite it?', $module);
                $args = array(
                    'tmpFile' => $tmpFile,
                    'name' => $fileName,
                    'path' => "$path/$fileName"
                );
                $actionYes = MUtil::getAjaxAction('confirmOverwrite', $args);

                MPopup::confirm($message, _M('Confirm overwrite', $module), $actionYes, 'mpopup.remove();');
                return;
            }

            if ( copy($tmpFile, "$path/$fileName") )
            {
                unlink($tmpFile);
                $uploadedFiles[] = array(
                    'name' => $fileName,
                    'path' => "$path/$fileName"
                );
            }
        }

        return $uploadedFiles;
    }

    public function submit_button_click($args)
    {
        $MIOLO = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();

        $path = $MIOLO->getConf('home.modules') . "/$module/templates";
        $uploadedFiles = $this->uploadFiles($path);

        if ( is_array($uploadedFiles) && count($uploadedFiles) == 1 )
        {
            new MMessageSuccess(_M('File @1 uploaded successfully', $module, $uploadedFiles[0]['name']));
        }
        elseif ( count($uploadedFiles) > 1 )
        {
            $files = array();
            foreach( $uploadedFiles as $uploadedFile )
            {
                $files[] = $uploadedFile['name'];
            }
            $files = implode(', ', $files);
            new MMessageSuccess(_M('Files @1 uploaded successfully', $module, $files));
        }
        elseif ( is_array($uploadedFiles) )
        {
            new MMessageWarning(_M('No files uploaded', $module));
        }

        $control = $this->getTemplateList();
        $this->setResponse($control, 'cTemplate', true);
    }
}

?>
